@@ -420,6 +420,7 @@ def sync(
     + delete: delete remote files not present locally
     + exclude: string or list/tuple of strings to match & exclude files (eg *.pyc)
     + exclude_dir: string or list/tuple of strings to match & exclude directories (eg node_modules)
+    + add_deploy_dir: interpret src as relative to deploy directory instead of current directory
 
     Example:
 
@@ -433,7 +434,7 @@ def sync(
         )
     '''
 
-    # If we don't enforce the source ending with /, remote_dirname below might start with
+    # If we don't enforce the source ending with /, remote_dirpath below might start with
     # a /, which makes the os_path.join cut off the destination bit.
     if not src.endswith(os_path.sep):
         src = '{0}{1}'.format(src, os_path.sep)
@@ -458,18 +459,20 @@ def sync(
 
     put_files = []
     ensure_dirnames = []
-    for dirname, _, filenames in walk(src):
-        remote_dirname = dirname.replace(src, '')
+    for dirpath, dirnames, filenames in walk(src, topdown=True):
+        remote_dirpath = dirpath.replace(src, '')
 
-        # Should we exclude this dir?
-        if exclude_dir and any(fnmatch(remote_dirname, match) for match in exclude_dir):
-            continue
+        # Filter excluded dirs
+        for child_dir in dirnames:
+            child_path = os_path.join(remote_dirpath, child_dir)
+            if exclude_dir and any(fnmatch(child_path, match) for match in exclude_dir):
+                dirnames.remove(child_dir)
 
-        if remote_dirname:
-            ensure_dirnames.append((remote_dirname, get_path_permissions_mode(dirname)))
+        if remote_dirpath:
+            ensure_dirnames.append((remote_dirpath, get_path_permissions_mode(dirpath)))
 
         for filename in filenames:
-            full_filename = os_path.join(dirname, filename)
+            full_filename = os_path.join(dirpath, filename)
 
             # Should we exclude this file?
             if exclude and any(fnmatch(full_filename, match) for match in exclude):
@@ -481,7 +484,7 @@ def sync(
                 # Join remote as unix like
                 '/'.join(
                     item for item in
-                    (dest, remote_dirname, filename)
+                    (dest, remote_dirpath, filename)
                     if item
                 ),
             ))
@@ -501,9 +504,9 @@ def sync(
     )
 
     # Ensure any remote dirnames
-    for dirname, dir_mode in ensure_dirnames:
+    for dirpath, dir_mode in ensure_dirnames:
         yield directory(
-            '/'.join((dest, dirname)),
+            '/'.join((dest, dirpath)),
             user=user, group=group, mode=dir_mode,
             state=state, host=host,
         )
